/**
  *  \file client/dlg-subject.cc
  */

#include "client/dlg-subject.h"
#include "ui/layout.h"
#include "gfx/palette.h"
#include "util/nls.h"
#include "util/misc.h"
#include "ui/widgets.h"

/*! \class WSubjectList
    \brief Message Subject List

    This widget displays a list of all message subjects of a
    GMessagebox. */

/** Constructor.
    \param id      Widget Id
    \param mbox    Mailbox box to display
    \param filter  Filter to apply to mailbox */
WSubjectList::WSubjectList(int id, GMessagebox& mbox, GMessageFilter& filter)
    : UIListbox(id, 20, 20 * font_ems[FONT_NORMAL], 0),
      mbox(mbox),
      filter(filter),
      groups()
{
    createGroupList();
}

WSubjectList::~WSubjectList()
{ }

/** Create the group list. */
void
WSubjectList::createGroupList()
{
    std::vector<Group> new_groups;

    int this_group_start = -1;
    string_t this_group_heading;

    for (int i = 0, max = mbox.getCount(); i < max; ++i) {
        string_t this_msg_heading = mbox.getHeading(i);
        if (this_msg_heading != this_group_heading) {
            if (this_group_start >= 0)
                new_groups.push_back(Group(this_group_start, i - this_group_start));
            this_group_heading = this_msg_heading;
            this_group_start = i;
        }
    }
    if (this_group_start >= 0)
        new_groups.push_back(Group(this_group_start, mbox.getCount() - this_group_start));

    new_groups.swap(groups);
    setSize(groups.size());

    // FIXME: redraw, reposition
}

/** Get selected message. Since each row can represent multiple
    messages, this returns the first message of the row. */
int
WSubjectList::getSelectedMessage() const
{
    if (getCurrent() < groups.size()) {
        return groups[getCurrent()].first_msg;
    } else {
        return 0;
    }
}

/** Select a message. Selects the row which contains the message. */
void
WSubjectList::selectMessage(int n)
{
    for (std::vector<Group>::size_type i = 0; i < groups.size(); ++i) {
        if (groups[i].first_msg >= n && n - groups[i].first_msg < groups[i].count) {
            scrollTo(i, 1);
            break;
        }
    }
}

void
WSubjectList::drawPart(GfxCanvas& can, int from, int to)
{
    for (int i = from; i < to; ++i) {
        const int index = getTop() + i;
        if (index >= groups.size()) {
            drawEntry(can, UIColor::tc_Static, i, "");
        } else {
            // draw by hand. FIXME: this is a copy of drawEntry.
            const int real_y = getItemHeight()*i + extent.y + getHeaderHeight();
            const int divi = 3*font_ems[FONT_NORMAL];
            UISkin& s = getSkin();
            GfxParameters& p = s;

            p.just_y = 0;

            const UIColorSet& scheme = (index != getCurrent() ? GRAY_COLOR_SET : hasState(st_Focused) ? BLACK_COLOR_SET : BRIGHT_GRAY_COLOR_SET);
            p.bgcolor = standard_colors[scheme[UIColor::tc_Background]];
            p.color   = standard_colors[scheme[UIColor::tc_Static]];
            p.font    = fonts[FONT_NORMAL];

            p.just_x = 2;
            outTextF(can, p, extent.x + divi, real_y, divi, numToString(groups[index].count));
            p.just_x = 0;
            outTextF(can, p, extent.x + divi, real_y, extent.w - divi,
                     " " + mbox.getHeading(groups[index].first_msg));
        }
    }
}

/** Do a standard dialog.
    \return nonzero if user chose OK */
int
WSubjectList::doStandardDialog()
{
    UIWindow window(_("Message Subjects"), UIWindow::BLUE_WINDOW, COLOR_GRAY);
    window.setLayout(UIVBoxLayout::instance5);

    UIGroup group_buttons(UIHBoxLayout::instance5);
    UIButton button_ok(1, _("OK"), SDLK_RETURN, UIButton::aStop);
    UIButton button_cancel(0, _("Cancel"), SDLK_ESCAPE, UIButton::aStop);
    UISpacer spacer;
    group_buttons.add(button_ok);
    group_buttons.add(button_cancel);
    group_buttons.add(spacer);

    window.add(*this);
    window.add(group_buttons);
    window.pack();

    setFocus();

    ui_root->centerWidget(window);
    ui_root->add(window);
    int result = ui_root->run();
    ui_root->close(window);

    return result;
}
