/**
  *  \file client/playerscr.cc
  *  \brief Player Main Screen
  *
  *  Main screen, user interface and multiplexing to other control screens.
  *  PCC 1.x source: cc.pas, scrmux.pas.
  *
  *  (c) 2001-2006 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include <iostream>
#include <SDL.h>
#include "client/screens.h"
#include "client/chartusr.h"
#include "client/cscreen.h"
#include "client/dlg-list.h"
#include "client/playerscr.h"
#include "client/vcrplay.h"
#include "game/global.h"
#include "game/objlist.h"
#include "game/planet.h"
#include "game/ship.h"
#include "gfx/font.h"
#include "gfx/gfxcompl.h"
#include "gfx/palette.h"
#include "ui/layout.h"
#include "ui/listbox.h"
#include "ui/ui.h"
#include "ui/util.h"
#include "ui/widgets.h"
#include "util/nls.h"
#include "cpluslib/holder.h"
#include "client/dlg-mbox.h"
#include "cpluslib/format.h"
#include "client/scr-sim.h"
#include "game/sim-game.h"

class WPixmapButton : public UIButton {
    Ptr<GfxPixmap> pix;
 public:
    WPixmapButton(int aid, string_t aname, UIKey akey, Ptr<GfxPixmap> apix);
    void draw(GfxCanvas& can, UpdateInfo& ui);
    void getLayoutInfo(LayoutInfo& info);
};

WPixmapButton::WPixmapButton(int aid, string_t aname,
                             UIKey akey, Ptr<GfxPixmap> apix)
    : UIButton(aid, aname, akey), pix(apix)
{ }

void
WPixmapButton::draw(GfxCanvas& can, UpdateInfo& ui)
{
    GfxParameters parm;
    int text_height = fonts[FONT_SMALL]->getTextHeight(getName());
    if (pix && extent.h > text_height-2)
        pix->blit(can, extent.x + 1, extent.y + 1,
                  GfxRect(0, 0, extent.w - 2, extent.h - text_height - 2));
    parm.color = standard_colors[COLOR_WHITE];
    parm.bgcolor = standard_colors[COLOR_BLACK];
    parm.font = fonts[FONT_SMALL];
    outTextF(can, parm,
             extent.x + 1, extent.y + extent.h - text_height - 1,
             extent.w - 2, getName());
    drawFrameDown(can, extent);
    ui.update(extent);
}

void
WPixmapButton::getLayoutInfo(LayoutInfo& info)
{
    GfxPoint pt (fonts[FONT_SMALL]->getTextWidth(getName()) + 2,
                 fonts[FONT_SMALL]->getTextHeight(getName()) + 2);
    if (pix) {
        pt.x = pix->getWidth() + 2;
        pt.y += pix->getHeight();
    }
    info.flags = 0;
    info.min_size = info.pref_size = pt;
}

/***************************** WPlayerScreen *****************************/

class WPlayerScreen : public UIGroup {
    Ptr<GfxPixmap> pix;
    static UIKeyTranslator::Init keys[];
    ObjectHolder holder;
    UIHBoxLayout layout;
    UIVBoxLayout vlayout;
 public:
    WPlayerScreen();
    void drawInterior(GfxCanvas& can, UpdateInfo& ui);
    bool handleEvent(const UIEvent& e, bool second_pass);
};

UIKeyTranslator::Init
WPlayerScreen::keys[] = {
    { SDLK_ESCAPE, cm_Escape }
};

WPlayerScreen::WPlayerScreen()
    : UIGroup(), layout(5, 5), vlayout(5, 5)
{
    setLayout(layout);
    setExtent(GfxRect(ui_screen->getSystemSurface()));
    pix = requestPixmap("bgtile");
    UIGroup& g = holder.add(new UIGroup(vlayout));
    g.add(holder.add(new WPixmapButton(1, _("F1 - Starships"), SDLK_F1,
                             requestPixmap("menu.ship"))));
    g.add(holder.add(new WPixmapButton(2, _("F2 - Planets"), SDLK_F2,
                             requestPixmap("menu.planet"))));
    g.add(holder.add(new WPixmapButton(3, _("F3 - Starbases"), SDLK_F3,
                             requestPixmap("menu.base"))));
    g.add(holder.add(new WPixmapButton(4, _("F4 - Starchart"), SDLK_F4,
                             requestPixmap("menu.chart"))));
    g.add(holder.add(new UISpacer()));
    add(g);
    g.pack();

    UIGroup& fgroup = holder.add(new UIGroup(holder.add(new UIFlowLayout(3, true))));
    fgroup.add(holder.add(new UIButton(_("B - Battle Simulator"), 'b')));
    fgroup.add(holder.add(new UIButton(_("M - Messages"), 'm')));
    fgroup.add(holder.add(new UIButton(_("W - Write Message"), 'w')));
    fgroup.add(holder.add(new UIButton(_("V - Combat Recorder"), 'v')));
    fgroup.add(holder.add(new UIButton(_("ESC - Exit"), SDLK_ESCAPE)));

    UIGroup& vgroup = holder.add(new UIGroup(vlayout));
    vgroup.add(holder.add(new UISpacer()));
    vgroup.add(fgroup);
    add(vgroup);

    doLayout();
    add(holder.add(new UIKeyTranslator(keys, countof(keys))));
}

void
WPlayerScreen::drawInterior(GfxCanvas& can, UpdateInfo& ui)
{
    drawTiledArea(can, GfxRect(extent.x + 1, extent.y + 1,
                               extent.w - 2, extent.h - 2),
                  pix, standard_colors[COLOR_DARK], 16);
    drawFrameUp(can, extent);

    GfxParameters parm;
    parm.font  = fonts[FONT_TITLE];
    parm.color = standard_colors[COLOR_WHITE];

    outText(can, parm, 200,  20, getFullName(getPlayerId()));
    outText(can, parm, 200,  60, format(_("Turn %d"), getDisplayedTurn().getTurnNumber()));
    outText(can, parm, 200,  80, format(_("%d message%!1{s%}"), getDisplayedTurn().getInbox().getCount()));
    outText(can, parm, 200, 100, format(_("%d combat recording%!1{s%}"), getDisplayedTurn().getVcrs().getCount()));

    ui.update(extent);
}

/** Play VCRs. */
static void
playVcrs(GVcrDatabase& db)
{
    static const UIKeyTranslator::Init keys[] = {
        { SDLK_UP,      cm_VCR_Prev },
        { SDLK_DOWN,    cm_VCR_Next },
        { vk_WheelDown, cm_VCR_Next },
        { vk_WheelUp,   cm_VCR_Prev },
        { vk_Quit,      cm_Escape }
    };

    // HACK HACK HACK
    GClassicVcrDatabase* cdb = dynamic_cast<GClassicVcrDatabase*>(&db);
    if (!cdb)
        return;

    ObjectHolder holder;
    UIWindow w(_("Visual Combat Recorder"), UIWindow::BLUE_WINDOW, COLOR_GRAY);

    UIGroup but(UIHBoxLayout::instance5);
    but.add(holder.add(new UIButton(cm_VCR_Prev, "\036", '-')));
    but.add(holder.add(new UIButton(cm_VCR_Next, "\037", '+')));
    but.add(holder.add(new UIButton(cm_Enter, _("ENTER - Play"), SDLK_RETURN)));
    but.add(holder.add(new UISpacer()));
    but.add(holder.add(new UIButton(cm_Escape, _("ESC - Exit"), SDLK_ESCAPE, UIButton::aStop)));

    WVcrSelector sel(*cdb);
    w.setLayout(UIVBoxLayout::instance5);
    w.add(sel);
    w.add(but);
    w.add(holder.add(new UIKeyTranslator(keys, countof(keys))));
    w.pack();
    ui_root->centerWidget(w);
    ui_root->add(w);
    ui_root->run();
    ui_root->close(w);
}

// #include "gfx/clip.h"
// void
// testEffect()
// {
//     Ptr<GfxPixmap> pix = requestPixmap(ResId("light_brushed"));
//     Ptr<GfxPixmap> bg = new GfxPixmap(320, 240, getPixelFormatOfSurface(ui_screen->getSystemSurface()));
//     GfxOffset ofs(-160, -120, *bg);
//     UIWidget::UpdateInfo ui(*bg);
//     ui_root->draw(ofs, ui);
//     ui.doUpdates();

//     // blit cannot yet do alpha
//     for (int y = 0; y < 240; ++y) {
//         for (int x = 0; x < 320; ++x) {
//             int alpha = 192;
//             uint32 color = pix->getPixel(x % pix->getWidth(), y % pix->getHeight());
//             if (x == 0 || y == 0)
//                 color = 0xFFFFFF;
//             if (x == 319 || y == 239)
//                 alpha = 0x888888;
//             bg->drawPixel(x, y, color, alpha);
//         }
//     }

//     // draw it
//     bg->blit(*ui_screen, 160, 120);
//     ui_screen->update(GfxRect(160, 120, 320, 240));

//     // wait
//     UIEvent e;
//     do
//         getEvent(e);
//     while (e.type != ev_Key);
// }

bool
WPlayerScreen::handleEvent(const UIEvent& e, bool second_pass)
{
    if (e.type == ev_Command) {
        switch(e.cmd.code) {
         case cm_Escape:
            stop(-1);
            return true;
         case 1:
         case 2:
         case 3:
         case 6:
         case 10:
            doMainLoop(-e.cmd.code);
            return true;
         case 4:
            doMainLoop(4);
            return true;
        }
    } else if (e.type == ev_Key) {
        if (e.key.code == 'v') {
            if (getDisplayedTurn().getVcrs().getCount())
                playVcrs(getDisplayedTurn().getVcrs());
            return true;
        } else if (e.key.code == 'm') {
            doMainLoop(doInboxScreen());
            return true;
        } else if (e.key.code == 'w') {
            doMainLoop(doSendOrReviseMessage());
            return true;
        } else if (e.key.code == 'b') {
            GSimulatorRealGameInterface rgi(getDisplayedTurn().getCurrentUniverse());
            doMainLoop(doSimulator(rgi));
            return true;
        }
    }
    return UIGroup::handleEvent(e, second_pass);
}

/***************************** Main Routines *****************************/

int
doPlayerScreen()
{
    WPlayerScreen ps;
    ui_root->add(ps);
    int rv = ui_root->run();
    ui_root->close(ps);
    return rv;
}

/** Main loop. Giant state machine. (cc.pas::RunScreen in PCC 1.x) */
int
doMainLoop(int state)
{
    while (1) {
        if (state < 0) {
            state = -state;
            if (int i = doSelectObject(state))
                getObjectType(state).setCurrent(i);
            else
                return 0;
        }

        switch(state) {
         case 1:
            state = doShipScreen();
            break;
         case 2:
            state = doPlanetScreen();
            break;
         case 3:
            state = doBaseScreen();
            break;
         case 4:
            state = doStarchart();
            break;
         default:
            return state;
        }
    }
}

/** Get type associated with a screen number. Screen numbers are function
    key numbers for the appropriate screens, without formal \#defines. */
GObjectType&
getObjectType(int screen, GGameTurn& trn)
{
    switch (screen) {
     case 1: case 11:
        return trn.getShipType();
     case 2: case 12:
        return trn.getPlanetType();
     case 3: case 13:
        return trn.getBaseType();
        // FIXME: fleets
     default:
        FAIL();
    }
}

/** Get type associated with a screen number. */
GObjectType&
getObjectType(int screen)
{
    return getObjectType(screen, getDisplayedTurn());
}

/** Do object selection window for a particular screen Id.
    \returns chosen Id or 0 */
int
doSelectObject(int screen)
{
    switch (screen) {
     case 1: case 11:
        return chooseShip();
     case 2: case 12:
        return choosePlanet();
     case 3: case 13:
        return chooseBase();
        // FIXME: ship history, fleets
     default:
        return 0;
    }
}

/** Do a control screen. Does selection window and posts a cm_CS_GotoCS
    if needed. */
void
doScreen(int screen_id)
{
    if (int i = doSelectObject(screen_id))
        putCommand(cm_CS_GotoCS, screen_id, i);
}
